database dbwork {

    data file d01 = "dbwork.d01" contains ref, norm;
    data file d02 = "dbwork.d02" contains fwd_ref;
    key  file k01 = "dbwork.k01" contains src_ref, crc_set;

    /* The 'ref' record stores the source db_addr of a record in 'src'.
       The 'set' value is an index into the set mapping table identifying
       the set type through which the record was referenced.  Thus each
       source record may have a 'ref' record created for each which it is
       involved in.
    */
    record ref {
	db_addr	src_dba;
	int	src_set;
	short	position;
	db_addr	dest_dba;
	compound unique key src_ref {
		src_dba;
		src_set;
		position;
	}
    }

    /* The 'fwd_ref' record is created and connected to a 'ref' record 
       through 'fwd_refs' when a reference is made in the dest db to a
       record that has not yet been transfered to the dest db.  When the
       record is transfered, its actual db_addr may be placed into the
       address pointed to by 'ref_addr'.
    */
    record fwd_ref {
	int n_refs;
	struct {
	    db_addr ref_dba;
	    int ref_offset;
	} ref_addr[10];
    }

    set fwd_refs {
	order last;
	owner ref;
	member fwd_ref;
    }


    /* Normalized sets have unique owner records.  When a new owner of
       a normalized set is created, it must first be verified to be
       unique.  If another identical record has already been created, then
       it will be used instead of the current one.  Each new record will
       be checked against the old ones by creating a checksum of the contents
       and looking for that checksum in the 'norm' record type.  If the 
       checksums match, the first record will be read, and if the full
       contents match, then the current record will be ignored; the existing
       one will be used in set connections.
    */
    record norm {
	long crc;
	int norm_set;
	db_addr norm_rec;
	compound key crc_set {
	    crc;
	    norm_set;
	}
    }

}
